package com.azure.ai.openai.usage;

import com.azure.ai.openai.OpenAIClient;
import com.azure.ai.openai.OpenAIClientBuilder;
import com.azure.ai.openai.models.*;
import com.azure.core.credential.AzureKeyCredential;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.RestTemplate;

import java.util.*;

@RestController
public class PromptController {
    @Value("${azure.openai.api.key}")
    private String azureOpenaiKey;

    @Value("${azure.openai.endpoint}")
    private String endpoint;

    @Value("${azure.openai.deployment.model.id}")
    private String deploymentOrModelId;
    
    @Value("${coinmarketcap.api.key}")
    private String coinMarketCapApiKey;
    
    private final String CRYPTO_BASE_URL = "https://pro-api.coinmarketcap.com/v1/cryptocurrency/quotes/latest";
    
    private final RestTemplate restTemplate = new RestTemplate();
    private final ObjectMapper objectMapper = new ObjectMapper();

    @PostMapping("/answer")
    public List<String> getMethodName(@RequestBody PromptQuestion promptQuestion) {
        List<String> responseList = new ArrayList<>();
        try {
            OpenAIClient client = new OpenAIClientBuilder()
                    .endpoint(endpoint)
                    .credential(new AzureKeyCredential(azureOpenaiKey))
                    .buildClient();
            
            List<ChatMessage> messages = new ArrayList<>();
            messages.add(new ChatMessage(ChatRole.SYSTEM, 
                "You are an AI assistant that helps people find information about cryptocurrencies. " +
                "You can use the getCryptoPrice function to get latest information about cryptocurrencies."));
            messages.add(new ChatMessage(ChatRole.USER, getPrompt(promptQuestion)));

            // Define function for crypto price lookup
            List<FunctionDefinition> functions = new ArrayList<>();
            
            // Parameters for getCryptoPrice function
            Map<String, Object> symbolParams = new HashMap<>();
            symbolParams.put("type", "string");
            symbolParams.put("description", "The cryptocurrency symbol (e.g., BTC, ETH, XRP)");
            
            Map<String, Object> functionProperties = new HashMap<>();
            functionProperties.put("symbol", symbolParams);
            
            Map<String, Object> functionSchema = new HashMap<>();
            functionSchema.put("type", "object");
            functionSchema.put("required", List.of("symbol"));
            functionSchema.put("properties", functionProperties);
            
            // Create function definition
            functions.add(new FunctionDefinition("getCryptoPrice")
                    .setDescription("Get the latest price and information about a cryptocurrency")
                    .setParameters(functionSchema));
            
            // Set options with functions
            ChatCompletionsOptions options = new ChatCompletionsOptions(messages)
                    .setTemperature(0.7)
                    .setTopP(0.95)
                    .setMaxTokens(800)
                    .setFunctions(functions);
            
            ChatCompletions completions = client.getChatCompletions(deploymentOrModelId, options);
            
            for (ChatChoice choice : completions.getChoices()) {
                ChatMessage responseMessage = choice.getMessage();
                System.out.println(responseMessage.getContent());
                if (responseMessage.getFunctionCall() != null) {
                    // Handle function call
                    FunctionCall functionCall = responseMessage.getFunctionCall();
                    
                    System.out.println("Function Called: " + functionCall.getName());
                    System.out.println("Function Arguments: " + functionCall.getArguments());
                    
                    if ("getCryptoPrice".equals(functionCall.getName())) {
                        try {
                            // Parse arguments
                            JsonNode args = objectMapper.readTree(functionCall.getArguments());
                            String symbol = args.get("symbol").asText();
                            
                            // Call CoinMarketCap API
                            String cryptoData = getCryptoPriceData(symbol);
                            
                            // Add function call to message history
                            messages.add(responseMessage);
                            
                            // Add function result
                            messages.add(new ChatMessage(ChatRole.FUNCTION, cryptoData)
                                    .setName("getCryptoPrice"));
                            
                            // Get final response with the function results
                            ChatCompletionsOptions finalOptions = new ChatCompletionsOptions(messages)
                                    .setTemperature(0.7)
                                    .setMaxTokens(800);
                            
                            ChatCompletions finalCompletions = client.getChatCompletions(deploymentOrModelId, finalOptions);
                            
                            for (ChatChoice finalChoice : finalCompletions.getChoices()) {
                                responseList.add(finalChoice.getMessage().getContent().trim());
                            }
                        } catch (Exception e) {
                            responseList.add("Error processing cryptocurrency request: " + e.getMessage());
                        }
                    }
                } else {
                    // No function call, just return the content
                    responseList.add(responseMessage.getContent().trim());
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            responseList.add("Exception Occurred: " + ex.getMessage());
        }
        return responseList;
    }
    
    /**
     * Call CoinMarketCap API to get cryptocurrency data
     * @param symbol Cryptocurrency symbol (e.g., BTC)
     * @return JSON response as string
     */
    private String getCryptoPriceData(String symbol) {
        try {
            // Prepare headers with API key
            HttpHeaders headers = new HttpHeaders();
            headers.set("X-CMC_PRO_API_KEY", coinMarketCapApiKey);
            headers.set("Accept", "application/json");
            
            // Build URL with parameters
            String url = CRYPTO_BASE_URL + "?symbol=" + symbol.toUpperCase();
            
            // Make API request
            ResponseEntity<String> response = restTemplate.exchange(
                url, 
                HttpMethod.GET, 
                new HttpEntity<>(headers),
                String.class
            );
            
            // Process and return a simplified version of the data
            JsonNode rootNode = objectMapper.readTree(response.getBody());
            JsonNode dataNode = rootNode.path("data").path(symbol.toUpperCase());
            
            if (dataNode.isMissingNode()) {
                return "{\"error\": \"Cryptocurrency not found\"}";
            }
            
            // Extract the relevant information
            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put("name", dataNode.path("name").asText());
            resultMap.put("symbol", dataNode.path("symbol").asText());
            resultMap.put("price", dataNode.path("quote").path("USD").path("price").asDouble());
            resultMap.put("volume_24h", dataNode.path("quote").path("USD").path("volume_24h").asDouble());
            resultMap.put("percent_change_1h", dataNode.path("quote").path("USD").path("percent_change_1h").asDouble());
            resultMap.put("percent_change_24h", dataNode.path("quote").path("USD").path("percent_change_24h").asDouble());
            resultMap.put("percent_change_7d", dataNode.path("quote").path("USD").path("percent_change_7d").asDouble());
            resultMap.put("market_cap", dataNode.path("quote").path("USD").path("market_cap").asDouble());
            resultMap.put("last_updated", dataNode.path("quote").path("USD").path("last_updated").asText());
            
            return objectMapper.writeValueAsString(resultMap);
            
        } catch (Exception e) {
            e.printStackTrace();
            return "{\"error\": \"" + e.getMessage() + "\"}";
        }
    }

    private String getPrompt(PromptQuestion promptQuestion) {
        String input = promptQuestion.getQuestion().trim();        
        return input;
    }
}